/***************************************************************************
 *   Copyright (C) 2009 by Rico Rommel   *
 *   rico@bierrommel.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "qkm.h"
#include <QApplication>
#include <QTimer>
#include <math.h>
#include <QDebug>
#include <QPalette>


qkm::qkm(QWidget *parent)
 : QWidget(parent)
{
	MainLayout = new QVBoxLayout;
	SpeedLayout = new QHBoxLayout;

	currentSpeedLabel = new QSpeedGroup("v akt.", this);
	SpeedLayout->addWidget(currentSpeedLabel);

	maxSpeedLabel = new QSpeedGroup("v max", this);
	SpeedLayout->addWidget(maxSpeedLabel);

	ResetMaxSpeedButton = new QPushButton(QString::fromUtf8("v max\nzurück-\nsetzen"), this);
	SpeedLayout->addWidget(ResetMaxSpeedButton);
	connect(ResetMaxSpeedButton,SIGNAL(clicked()),this,SLOT(resetMaxSpeed()));


	CloseButton = new QPushButton(QString::fromUtf8("Schließen"), this);
	QPalette pal = CloseButton->palette();
	pal.setColor(QPalette::Button, Qt::red);
	CloseButton->setPalette( pal );
	CloseButton->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
	connect(CloseButton,SIGNAL(clicked()),this,SLOT(onCloseButtonClicked()));
	SpeedLayout->addWidget(CloseButton);

	MainLayout->addLayout(SpeedLayout);



	QString number;
	int n;
	for (n = 0; n < 3; n++) {
		number.sprintf("%d",n+1);
		counter[n] = new QCounterGroup(QString::fromUtf8("Zähler ") + number);
		MainLayout->addWidget(counter[n]);
		}


	setLayout(MainLayout);

	lastlat = -1;
	lastTime = 0;
	R = 6371;

	DevInterface = new QDBusInterface("org.freesmartphone.ogpsd","/org/freedesktop/Gypsy","org.freedesktop.Gypsy.Device",QDBusConnection::systemBus(), this);
	qDebug() << DevInterface->isValid();

	PosInterface = new QDBusInterface("org.freesmartphone.ogpsd","/org/freedesktop/Gypsy","org.freedesktop.Gypsy.Position",QDBusConnection::systemBus(), this);
	qDebug() << PosInterface->isValid();

	DevInterface->call("Start");
	connect(PosInterface,SIGNAL(PositionChanged(int,int,double,double,double)),this,SLOT(onPositionChanged(int, int, double, double, double)));

}


qkm::~qkm()
{
}




void qkm::onCloseButtonClicked()
{
	DevInterface->call("Stop");
   	qApp->quit();
}


void qkm::onPositionChanged(int f, int ts, double lat, double lon, double alt)
{

if ( ts - lastTime > 2) {
    	if (lastlat != -1) {
		double d = acos( sin( deg2rad( lastlat ) ) * sin( deg2rad( lat ) ) + cos( deg2rad( lastlat ) ) * cos( deg2rad( lat ) )*cos( deg2rad( lon - lastlon ) ) ) * R;
		
		int n;

		for (n= 0; n < 3; n++) counter[n]->update( d, ts - lastTime);

		double currentSpeed = d / (ts - lastTime) * 3600;
		currentSpeedLabel->setSpeed( currentSpeed );

		if (currentSpeed > maxSpeedLabel->getSpeed()) maxSpeedLabel->setSpeed( currentSpeed );

		}

	lastlat = lat;
	lastlon = lon;
	lastalt = alt;
	lastTime = ts;

}

}


double qkm::deg2rad( double deg )
{
    	double rad = M_PI * deg / 180;
	return rad;
}


/*!
    \fn qkm::resetMaxSpeed()
 */
void qkm::resetMaxSpeed()
{
    maxSpeedLabel->setSpeed( 0 );
}
